@@ -4,6 +4,7 @@
 
 #include "tepl-style-scheme-chooser-widget.h"
 #include <gtksourceview/gtksource.h>
+#include "tepl-utils.h"
 
 /**
  * SECTION:style-scheme-chooser-widget
@@ -45,21 +46,6 @@ G_DEFINE_TYPE_WITH_CODE (TeplStyleSchemeChooserWidget,
 			 G_IMPLEMENT_INTERFACE (GTK_SOURCE_TYPE_STYLE_SCHEME_CHOOSER,
 						gtk_source_style_scheme_chooser_interface_init))
 
-static void
-clear_list_box_foreach_cb (GtkWidget *child,
-			   gpointer   user_data)
-{
-	gtk_widget_destroy (child);
-}
-
-static void
-clear_list_box (GtkListBox *list_box)
-{
-	gtk_container_foreach (GTK_CONTAINER (list_box),
-			       clear_list_box_foreach_cb,
-			       NULL);
-}
-
 static void
 list_box_row_set_style_scheme (GtkListBoxRow        *list_box_row,
 			       GtkSourceStyleScheme *style_scheme)
@@ -159,25 +145,6 @@ tepl_style_scheme_chooser_widget_dispose (GObject *object)
 	G_OBJECT_CLASS (tepl_style_scheme_chooser_widget_parent_class)->dispose (object);
 }
 
-static void
-scroll_to_row (GtkListBox    *list_box,
-	       GtkListBoxRow *row)
-{
-	/* See also the call to gtk_container_set_focus_vadjustment() below. */
-	gtk_container_set_focus_child (GTK_CONTAINER (list_box), GTK_WIDGET (row));
-}
-
-static void
-scroll_to_selected_row (GtkListBox *list_box)
-{
-	GtkListBoxRow *selected_row = gtk_list_box_get_selected_row (list_box);
-
-	if (selected_row != NULL)
-	{
-		scroll_to_row (list_box, selected_row);
-	}
-}
-
 static void
 tepl_style_scheme_chooser_widget_map (GtkWidget *widget)
 {
@@ -188,7 +155,7 @@ tepl_style_scheme_chooser_widget_map (GtkWidget *widget)
 		GTK_WIDGET_CLASS (tepl_style_scheme_chooser_widget_parent_class)->map (widget);
 	}
 
-	scroll_to_selected_row (chooser->priv->list_box);
+	tepl_utils_list_box_scroll_to_selected_row (chooser->priv->list_box);
 }
 
 static void
@@ -265,7 +232,7 @@ tepl_style_scheme_chooser_widget_set_style_scheme (GtkSourceStyleSchemeChooser *
 		if (style_scheme_equal (cur_style_scheme, style_scheme))
 		{
 			gtk_list_box_select_row (chooser->priv->list_box, cur_list_box_row);
-			scroll_to_row (chooser->priv->list_box, cur_list_box_row);
+			tepl_utils_list_box_scroll_to_row (chooser->priv->list_box, cur_list_box_row);
 			break;
 		}
 	}
@@ -372,7 +339,7 @@ style_scheme_manager_notify_scheme_ids_cb (GtkSourceStyleSchemeManager  *manager
 
 	style_scheme_id = tepl_style_scheme_chooser_widget_get_style_scheme_id (chooser);
 
-	clear_list_box (chooser->priv->list_box);
+	tepl_utils_list_box_clear (chooser->priv->list_box);
 	populate_list_box (chooser);
 
 	/* Note that the style_scheme_id may no longer exist, in which case no
@@ -380,7 +347,7 @@ style_scheme_manager_notify_scheme_ids_cb (GtkSourceStyleSchemeManager  *manager
 	 */
 	tepl_style_scheme_chooser_widget_set_style_scheme_id (chooser, style_scheme_id);
 
-	scroll_to_selected_row (chooser->priv->list_box);
+	tepl_utils_list_box_scroll_to_selected_row (chooser->priv->list_box);
 
 	g_signal_handlers_unblock_by_func (chooser->priv->list_box,
 					   list_box_selected_rows_changed_cb,
@@ -410,7 +377,6 @@ static void
 tepl_style_scheme_chooser_widget_init (TeplStyleSchemeChooserWidget *chooser)
 {
 	GtkWidget *scrolled_window;
-	GtkAdjustment *vadjustment;
 
 	chooser->priv = tepl_style_scheme_chooser_widget_get_instance_private (chooser);
 
@@ -423,6 +389,7 @@ tepl_style_scheme_chooser_widget_init (TeplStyleSchemeChooserWidget *chooser)
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_set_hexpand (scrolled_window, TRUE);
 	gtk_widget_set_vexpand (scrolled_window, TRUE);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_IN);
 
 	/* Overlay scrolling gets in the way when trying to select the last row. */
 	gtk_scrolled_window_set_overlay_scrolling (GTK_SCROLLED_WINDOW (scrolled_window), FALSE);
@@ -432,8 +399,8 @@ tepl_style_scheme_chooser_widget_init (TeplStyleSchemeChooserWidget *chooser)
 	gtk_widget_show_all (scrolled_window);
 	gtk_container_add (GTK_CONTAINER (chooser), scrolled_window);
 
-	vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window));
-	gtk_container_set_focus_vadjustment (GTK_CONTAINER (chooser->priv->list_box), vadjustment);
+	tepl_utils_list_box_setup_scrolling (chooser->priv->list_box,
+					     GTK_SCROLLED_WINDOW (scrolled_window));
 
 	g_signal_connect (chooser->priv->list_box,
 			  "selected-rows-changed",
