@@ -11,6 +11,7 @@
 #include <string.h>
 #include "tepl-application-window.h"
 #include "tepl-icu.h"
+#include "tepl-pango.h"
 
 /**
  * SECTION:utils
@@ -300,6 +301,80 @@ tepl_utils_get_file_shortname (const gchar *filename)
 	return g_strndup (filename, get_extension_position (filename));
 }
 
+static gchar *
+get_home_dir_without_trailing_slash (const gchar *home_dir)
+{
+	gchar *utf8_home_dir;
+	gsize length;
+
+	if (home_dir == NULL)
+	{
+		return NULL;
+	}
+
+	utf8_home_dir = g_filename_to_utf8 (home_dir, -1, NULL, NULL, NULL);
+	if (utf8_home_dir == NULL)
+	{
+		return NULL;
+	}
+
+	length = strlen (utf8_home_dir);
+	if (length == 0)
+	{
+		g_free (utf8_home_dir);
+		return NULL;
+	}
+
+	if (utf8_home_dir[length - 1] == '/')
+	{
+		utf8_home_dir[length - 1] = '\0';
+	}
+
+	return utf8_home_dir;
+}
+
+/* Like tepl_utils_replace_home_dir_with_tilde() but with an additional home_dir
+ * parameter, for unit tests.
+ */
+gchar *
+_tepl_utils_replace_home_dir_with_tilde_with_param (const gchar *filename,
+						    const gchar *home_dir)
+{
+	gchar *home_dir_without_trailing_slash;
+	gchar *home_dir_with_trailing_slash;
+	gchar *ret;
+
+	g_return_val_if_fail (filename != NULL, NULL);
+
+	home_dir_without_trailing_slash = get_home_dir_without_trailing_slash (home_dir);
+	if (home_dir_without_trailing_slash == NULL)
+	{
+		return g_strdup (filename);
+	}
+
+	home_dir_with_trailing_slash = g_strdup_printf ("%s/", home_dir_without_trailing_slash);
+
+	if (g_str_equal (filename, home_dir_without_trailing_slash) ||
+	    g_str_equal (filename, home_dir_with_trailing_slash))
+	{
+		ret = g_strdup ("~");
+		goto out;
+	}
+
+	if (g_str_has_prefix (filename, home_dir_with_trailing_slash))
+	{
+		ret = g_strdup_printf ("~/%s", filename + strlen (home_dir_with_trailing_slash));
+		goto out;
+	}
+
+	ret = g_strdup (filename);
+
+out:
+	g_free (home_dir_without_trailing_slash);
+	g_free (home_dir_with_trailing_slash);
+	return ret;
+}
+
 /**
  * tepl_utils_replace_home_dir_with_tilde:
  * @filename: the filename.
@@ -310,48 +385,10 @@ tepl_utils_get_file_shortname (const gchar *filename)
  * Returns: the new filename. Free with g_free().
  * Since: 4.4
  */
-/* This function comes from gedit. */
 gchar *
 tepl_utils_replace_home_dir_with_tilde (const gchar *filename)
 {
-	gchar *tmp;
-	gchar *home;
-
-	g_return_val_if_fail (filename != NULL, NULL);
-
-	/* Note that g_get_home_dir returns a const string */
-	tmp = (gchar *) g_get_home_dir ();
-
-	if (tmp == NULL)
-	{
-		return g_strdup (filename);
-	}
-
-	home = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
-	if (home == NULL)
-	{
-		return g_strdup (filename);
-	}
-
-	if (g_str_equal (filename, home))
-	{
-		g_free (home);
-		return g_strdup ("~");
-	}
-
-	tmp = home;
-	home = g_strdup_printf ("%s/", tmp);
-	g_free (tmp);
-
-	if (g_str_has_prefix (filename, home))
-	{
-		gchar *res = g_strdup_printf ("~/%s", filename + strlen (home));
-		g_free (home);
-		return res;
-	}
-
-	g_free (home);
-	return g_strdup (filename);
+	return _tepl_utils_replace_home_dir_with_tilde_with_param (filename, g_get_home_dir ());
 }
 
 static void
@@ -390,9 +427,8 @@ tepl_utils_decode_uri (const gchar  *uri,
 		       gchar       **path)
 {
 	/* Largely copied from glib/gio/gdummyfile.c: _g_decode_uri().
-	 * This functionality is currently not in GLib/GIO, so for now we
-	 * implement it ourselves. See:
-	 * https://bugzilla.gnome.org/show_bug.cgi?id=555490
+	 * TODO: it's now available in GLib with GUri, so deprecate/remove this
+	 * function.
 	 */
 
 	const char *p, *in, *hier_part_start, *hier_part_end;
@@ -798,6 +834,294 @@ tepl_utils_show_warning_dialog (GtkWindow   *parent,
 	gtk_widget_show (dialog);
 }
 
+static void
+list_box_clear_foreach_cb (GtkWidget *child,
+			   gpointer   user_data)
+{
+	gtk_widget_destroy (child);
+}
+
+/**
+ * tepl_utils_list_box_clear:
+ * @list_box: a #GtkListBox.
+ *
+ * Removes all rows of @list_box, to obtain an empty #GtkListBox.
+ *
+ * Since: 6.0
+ */
+void
+tepl_utils_list_box_clear (GtkListBox *list_box)
+{
+	g_return_if_fail (GTK_IS_LIST_BOX (list_box));
+
+	gtk_container_foreach (GTK_CONTAINER (list_box),
+			       list_box_clear_foreach_cb,
+			       NULL);
+}
+
+/**
+ * tepl_utils_list_box_setup_scrolling:
+ * @list_box: a #GtkListBox.
+ * @scrolled_window: a #GtkScrolledWindow.
+ *
+ * Setup vertical scrolling between @list_box and @scrolled_window, to be able
+ * to use tepl_utils_list_box_scroll_to_row() afterwards.
+ *
+ * This function is intended to be called only once per #GtkListBox, when
+ * initializing the @list_box and @scrolled_window widgets.
+ *
+ * Since: 6.0
+ */
+void
+tepl_utils_list_box_setup_scrolling (GtkListBox        *list_box,
+				     GtkScrolledWindow *scrolled_window)
+{
+	GtkAdjustment *vadjustment;
+
+	g_return_if_fail (GTK_IS_LIST_BOX (list_box));
+	g_return_if_fail (GTK_IS_SCROLLED_WINDOW (scrolled_window));
+
+	vadjustment = gtk_scrolled_window_get_vadjustment (scrolled_window);
+	gtk_container_set_focus_vadjustment (GTK_CONTAINER (list_box), vadjustment);
+}
+
+/**
+ * tepl_utils_list_box_scroll_to_row:
+ * @list_box: a #GtkListBox.
+ * @row: a #GtkListBoxRow.
+ *
+ * Scrolls to a specific #GtkListBoxRow.
+ *
+ * Before using this function, tepl_utils_list_box_setup_scrolling() must have
+ * been called.
+ *
+ * Since: 6.0
+ */
+void
+tepl_utils_list_box_scroll_to_row (GtkListBox    *list_box,
+				   GtkListBoxRow *row)
+{
+	g_return_if_fail (GTK_IS_LIST_BOX (list_box));
+	g_return_if_fail (GTK_IS_LIST_BOX_ROW (row));
+
+	gtk_container_set_focus_child (GTK_CONTAINER (list_box), GTK_WIDGET (row));
+}
+
+/**
+ * tepl_utils_list_box_scroll_to_selected_row:
+ * @list_box: a #GtkListBox.
+ *
+ * Calls tepl_utils_list_box_scroll_to_row() on the row returned by
+ * gtk_list_box_get_selected_row(). This function assumes that there is either
+ * zero or one selected row.
+ *
+ * Before using this function, tepl_utils_list_box_setup_scrolling() must have
+ * been called.
+ *
+ * Since: 6.0
+ */
+void
+tepl_utils_list_box_scroll_to_selected_row (GtkListBox *list_box)
+{
+	GtkListBoxRow *selected_row;
+
+	g_return_if_fail (GTK_IS_LIST_BOX (list_box));
+
+	selected_row = gtk_list_box_get_selected_row (list_box);
+	if (selected_row != NULL)
+	{
+		tepl_utils_list_box_scroll_to_row (list_box, selected_row);
+	}
+}
+
+/**
+ * tepl_utils_list_box_get_row_at_index_with_filter:
+ * @list_box: a #GtkListBox.
+ * @index: the index of the row, starting at 0. The index is among the filtered
+ *   rows only.
+ * @filter_func: (scope call): non-%NULL callback function.
+ * @user_data: user data passed to @filter_func.
+ *
+ * This function has the same semantics as gtk_list_box_get_row_at_index(), but
+ * it takes into account only the rows for which @filter_func returns %TRUE.
+ *
+ * As an example, if @index is 0, it returns the first #GtkListBoxRow for which
+ * @filter_func returns %TRUE.
+ *
+ * Returns: (transfer none) (nullable): the child #GtkListBoxRow or %NULL.
+ * Since: 6.0
+ */
+GtkListBoxRow *
+tepl_utils_list_box_get_row_at_index_with_filter (GtkListBox           *list_box,
+						  gint                  index,
+						  GtkListBoxFilterFunc  filter_func,
+						  gpointer              user_data)
+{
+	GList *all_rows;
+	GList *l;
+	gint remaining_rows_to_find = index + 1;
+	GtkListBoxRow *ret = NULL;
+
+	g_return_val_if_fail (GTK_IS_LIST_BOX (list_box), NULL);
+	g_return_val_if_fail (filter_func != NULL, NULL);
+
+	if (index < 0)
+	{
+		return NULL;
+	}
+
+	all_rows = gtk_container_get_children (GTK_CONTAINER (list_box));
+
+	for (l = all_rows; l != NULL; l = l->next)
+	{
+		GtkListBoxRow *cur_row = GTK_LIST_BOX_ROW (l->data);
+
+		if (filter_func (cur_row, user_data))
+		{
+			remaining_rows_to_find--;
+
+			if (remaining_rows_to_find == 0)
+			{
+				ret = cur_row;
+				break;
+			}
+		}
+	}
+
+	g_list_free (all_rows);
+	return ret;
+}
+
+/**
+ * tepl_utils_list_box_get_filtered_children:
+ * @list_box: a #GtkListBox.
+ * @filter_func: (scope call): non-%NULL callback function.
+ * @user_data: user data passed to @filter_func.
+ * @n_filtered_children: (out) (optional): location to store the number of
+ *   #GtkListBoxRow's present in the returned array, without counting the
+ *   terminating %NULL.
+ *
+ * Gets an array of all the #GtkListBoxRow childen of @list_box for which
+ * @filter_func returns %TRUE. The elements in the array are sorted by
+ * increasing index order (as returned by gtk_list_box_row_get_index()).
+ *
+ * Returns: (array zero-terminated=1) (element-type GtkListBoxRow) (transfer container) (nullable):
+ *   a %NULL-terminated array of #GtkListBoxRow objects, or %NULL. Free with
+ *   g_free() when no longer needed.
+ * Since: 6.0
+ */
+GtkListBoxRow **
+tepl_utils_list_box_get_filtered_children (GtkListBox           *list_box,
+					   GtkListBoxFilterFunc  filter_func,
+					   gpointer              user_data,
+					   gint                 *n_filtered_children)
+{
+	GPtrArray *filtered_rows;
+	GList *all_rows;
+	GList *l;
+
+	g_return_val_if_fail (GTK_IS_LIST_BOX (list_box), NULL);
+	g_return_val_if_fail (filter_func != NULL, NULL);
+
+	filtered_rows = g_ptr_array_new ();
+	all_rows = gtk_container_get_children (GTK_CONTAINER (list_box));
+
+	for (l = all_rows; l != NULL; l = l->next)
+	{
+		GtkListBoxRow *cur_row = GTK_LIST_BOX_ROW (l->data);
+
+		if (filter_func (cur_row, user_data))
+		{
+			g_ptr_array_add (filtered_rows, cur_row);
+		}
+	}
+
+	g_list_free (all_rows);
+
+	if (n_filtered_children != NULL)
+	{
+		*n_filtered_children = filtered_rows->len;
+	}
+
+	/* NULL-terminate the array, must be done *after* setting
+	 * *n_filtered_children, of course.
+	 */
+	g_ptr_array_add (filtered_rows, NULL);
+
+	return (GtkListBoxRow **) g_ptr_array_free (filtered_rows, FALSE);
+}
+
+/**
+ * tepl_utils_override_font:
+ * @widget: a #GtkWidget.
+ * @font_str: (nullable): a string representation of a #PangoFontDescription, or
+ *   %NULL to undo the effect of previous calls to this function on @widget.
+ *
+ * A replacement for gtk_widget_override_font(). Because
+ * gtk_widget_override_font() is deprecated but was useful.
+ *
+ * See pango_font_description_from_string() for a description of the format of
+ * the string representation for @font_str.
+ *
+ * This function uses tepl_pango_font_description_to_css() and applies the CSS
+ * to the #GtkStyleContext of @widget.
+ *
+ * Since: 6.0
+ */
+void
+tepl_utils_override_font (GtkWidget   *widget,
+			  const gchar *font_str)
+{
+	GtkStyleContext *style_context;
+	GtkCssProvider *css_provider;
+	PangoFontDescription *font_description;
+	gchar *css_declarations;
+	gchar *css_rule_set;
+
+#define FONT_CSS_PROVIDER_KEY "tepl-utils-override-font-css-provider-key"
+
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	style_context = gtk_widget_get_style_context (widget);
+	css_provider = g_object_get_data (G_OBJECT (widget), FONT_CSS_PROVIDER_KEY);
+
+	if (css_provider != NULL)
+	{
+		gtk_style_context_remove_provider (style_context, GTK_STYLE_PROVIDER (css_provider));
+		g_object_set_data (G_OBJECT (widget), FONT_CSS_PROVIDER_KEY, NULL);
+		css_provider = NULL;
+	}
+
+	if (font_str == NULL)
+	{
+		return;
+	}
+
+	font_description = pango_font_description_from_string (font_str);
+	g_return_if_fail (font_description != NULL);
+
+	css_declarations = tepl_pango_font_description_to_css (font_description);
+	css_rule_set = g_strdup_printf ("* {\n"
+					"    %s\n"
+				        "}\n",
+					css_declarations);
+
+	css_provider = gtk_css_provider_new ();
+	g_object_set_data_full (G_OBJECT (widget),
+				FONT_CSS_PROVIDER_KEY,
+				css_provider,
+				g_object_unref);
+
+	gtk_css_provider_load_from_data (css_provider, css_rule_set, -1, NULL);
+	gtk_style_context_add_provider (style_context,
+					GTK_STYLE_PROVIDER (css_provider),
+					TEPL_UTILS_STYLE_PROVIDER_PRIORITY_LIBRARY);
+
+	pango_font_description_free (font_description);
+	g_free (css_declarations);
+	g_free (css_rule_set);
+}
+
 /**
  * tepl_utils_binding_transform_func_smart_bool:
  * @binding: a #GBinding.
